

from __future__ import print_function

import glob
import re
import argparse
import os
import datetime
import sys


path_re = re.compile(r'cache/thread_(?P<title>[0-9a-f]+)/record/(?P<stamp>[0-9]+)_(?P<id>[0-9a-f]+)$', re.I)

class Record(object):
    def __init__(self, path):
        path = os.path.normpath(path)
        path = path.replace(os.sep, '/')
        m = path_re.search(path)
        if not m:
            return None

        self.path = path
        self.thread_key = 'thread_' + m.group('title')
        self.stamp = int(m.group('stamp'))
        self.id = m.group('id')


    @classmethod
    def all_records(cls, cache_dir):
        paths = glob.glob(os.path.join(cache_dir, './*/record/*'))
        records = (Record(p) for p in paths)
        return (r for r in records if r is not None)


def size_all(cachedir):
    size = 0
    for stat in glob.glob(os.path.join(cachedir, '*/size.stat')):
        with open(stat, 'r') as f:
            size += int(f.read().strip())
    return size


def count(records, tokey, isaccept=lambda r: True):
    result = {}
    for r in records:
        if not isaccept(r):
            continue

        key = tokey(r)
        if key is None:
            continue

        if key in result:
            result[key] += 1
        else:
            result[key] = 0

    return result


def daily_key(r):
    d = datetime.datetime.fromtimestamp(r.stamp)
    return d.year, d.month, d.day

daily_key.datestr = lambda t: '{}/{:02}/{:02}'.format(t[0], t[1], t[2]) 
daily_key.name = 'daily'

def weekly_key(r):
    d = datetime.datetime.fromtimestamp(r.stamp)
    return d.year, d.month, d.day // 7 * 7 + 1

weekly_key.datestr = lambda t: '{}/{:02}/{:02}'.format(t[0], t[1], t[2])
weekly_key.name = 'weekly'

def monthly_key(r):
    d = datetime.datetime.fromtimestamp(r.stamp)
    return d.year, d.month

monthly_key.datestr = lambda t: '{}/{:02}'.format(t[0], t[1])
monthly_key.name = 'monthly'




def aggr_span(cachedir, tokey):
    records = Record.all_records(cachedir)
    result = count(records, tokey=tokey)
    return result


def aggr_to_file(cachedir, tokey, hist=-1, csv=False, out=sys.stdout):
    result = aggr_span(cachedir, tokey)
    tohist = lambda c: '*' * (c // hist + 1)  if hist > 0 else c

    for t, c in sorted(result.items()):
        if csv:
            print('{},{}'.format(tokey.datestr(t), tohist(c)), file=out)
        else:
            print('{} {}'.format(tokey.datestr(t), tohist(c)), file=out)



def get_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument('cachedir', metavar='CACHE_DIR')
    behavior = parser.add_mutually_exclusive_group(required=True)

    behavior.add_argument('--daily', action='store_const', dest='type',
                          const=daily_key)
    behavior.add_argument('--monthly', action='store_const', dest='type',
                          const=monthly_key)
    behavior.add_argument('--weekly', action='store_const', dest='type',
                          const=weekly_key)

    parser.add_argument('--hist', nargs='?', type=int, default=-1, const=3)
    parser.add_argument('--csv', action='store_true')
    return parser


def main():
    parser = get_parser()
    args = parser.parse_args()

    hist = args.hist
    csv = args.csv
    aggr_to_file(args.cachedir, tokey=args.type, hist=hist, csv=csv, out=sys.stdout)


if __name__ == '__main__':
    main()
